@@ -67,6 +67,7 @@ void SemanticsListEngine::run()
     QString groupByCategoryType;
     QString groupByField;
     QString limitFilter;
+    int originalGenreLimit = 0;
     if (engineFilterList.count() != 0) {
         mediaType = engineFilterList.at(0);
         if (engineFilterList.filter("groupBy=").count() != 0) {
@@ -98,6 +99,14 @@ void SemanticsListEngine::run()
         }
         if (engineFilterList.filter("limit=").count() !=0) {
             limitFilter = engineFilterList.filter("limit=").at(0);
+            if (groupByField == "genre") {
+                originalGenreLimit = m_mediaListProperties.filterValue(limitFilter).trimmed().toInt();
+                int originalFilterIndex = engineFilterList.indexOf(limitFilter);
+                limitFilter = QString("%1%2%3").arg(m_mediaListProperties.filterField(limitFilter))
+                                               .arg(m_mediaListProperties.filterOperator(limitFilter))
+                                               .arg(m_mediaListProperties.filterValue(limitFilter).trimmed().toInt()*3);
+                engineFilterList.replace(originalFilterIndex, limitFilter);
+            }
         }
     }
     
@@ -195,7 +204,11 @@ void SemanticsListEngine::run()
                     if (!mediaItem.url.startsWith("nepomuk:/")) {
                         if ((ignoreZeros && mediaItem.fields["playCount"].toInt() > 0) ||
                             !ignoreZeros) {
-                            mediaList.append(mediaItem);
+                            if (groupByCategoryType == "AudioGenre") {
+                                addUniqueGenreGroup("playCount", mediaItem, &mediaList, originalGenreLimit);
+                            } else {
+                                mediaList.append(mediaItem);
+                            }
                         }
                     }
                 }
@@ -259,7 +272,11 @@ void SemanticsListEngine::run()
                     }
                     mediaItem.semanticComment = Utilities::wordsForTimeSince(mediaItem.fields["lastPlayed"].toDateTime());
                     if (!mediaItem.url.startsWith("nepomuk:/")) {
-                        mediaList.append(mediaItem);
+                        if (groupByCategoryType == "AudioGenre") {
+                            addUniqueGenreGroup("lastPlayed", mediaItem, &mediaList, originalGenreLimit);
+                        } else {
+                            mediaList.append(mediaItem);
+                        }
                     }
                 }
                 m_mediaListProperties.name = i18n("Recently Played");
@@ -283,8 +300,8 @@ void SemanticsListEngine::run()
                     if (!groupByResourceBinding.isEmpty()) {
                         bindings.append(groupByResourceBinding);
                     }
-                    bindings.append(MediaQuery::aggregateBinding(mediaVocabulary.ratingBinding(), MediaQuery::CountAverage));
-                    bindings.append(MediaQuery::aggregateBinding(mediaVocabulary.ratingBinding(), MediaQuery::Average));
+                    bindings.append(MediaQuery::aggregateBinding(mediaVocabulary.ratingBinding(), MediaQuery::Sum));
+                    bindings.append(MediaQuery::aggregateBinding(mediaVocabulary.ratingBinding(), MediaQuery::Count));
                 } 
                 query.select(bindings, MediaQuery::Distinct);
                 query.startWhere();
@@ -308,7 +325,7 @@ void SemanticsListEngine::run()
                     orderByBindings.append(mediaVocabulary.playCountBinding());
                     order.append(MediaQuery::Descending);
                 } else {
-                    orderByBindings.append(QString("%1_countavg").arg(mediaVocabulary.ratingBinding()));
+                    orderByBindings.append(QString("%1_sum").arg(mediaVocabulary.ratingBinding()));
                     order.append(MediaQuery::Descending);
                     order.append(MediaQuery::Descending);
                 }
@@ -327,13 +344,19 @@ void SemanticsListEngine::run()
                         mediaItem = Utilities::mediaItemFromNepomuk(res, m_mediaListProperties.lri);
                     } else {
                         mediaItem = Utilities::categoryMediaItemFromIterator(it, groupByCategoryType, m_mediaListProperties.lri);
-                        int rating = it.binding(QString("%1_avg").arg(mediaVocabulary.ratingBinding())).literal().toInt();
+                        int sum = it.binding(QString("%1_sum").arg(mediaVocabulary.ratingBinding())).literal().toInt();
+                        int count = it.binding(QString("%1_count").arg(mediaVocabulary.ratingBinding())).literal().toInt();
+                        int rating = sum/count;
                         mediaItem.fields["rating"] = rating;
                     }
                     if (!mediaItem.url.startsWith("nepomuk:/")) {
                         if ((ignoreZeros && mediaItem.fields["rating"].toInt() > 0) ||
                             !ignoreZeros) {
-                            mediaList.append(mediaItem);
+                            if (groupByCategoryType == "AudioGenre") {
+                                addUniqueGenreGroup("rating", mediaItem, &mediaList, originalGenreLimit);
+                            } else {
+                                mediaList.append(mediaItem);
+                            }
                         }
                     }
                 }
@@ -341,6 +364,51 @@ void SemanticsListEngine::run()
                 m_mediaListProperties.type = QString("Sources");
             }
         }
+        if (engineArg.toLower() == "recentlyadded") {
+            mediaList.clear();
+            if (!mediaType.isEmpty()) {
+                MediaQuery query;
+                QStringList bindings;
+                bindings.append(mediaVocabulary.mediaResourceBinding());
+                bindings.append("added");
+                query.select(bindings, MediaQuery::Distinct);
+                query.startWhere();
+                query.addCondition("graph ?g { ");
+                if (mediaType == "audio") {
+                    query.addCondition(mediaVocabulary.hasTypeAnyAudio(MediaQuery::Required));
+                } else if (mediaType == "video") {
+                    query.addCondition(mediaVocabulary.hasTypeAnyVideo(MediaQuery::Required));
+                }
+                query.addLRIFilterConditions(engineFilterList, mediaVocabulary);
+                query.addCondition("} ");
+                query.addCondition("?g nao:created ?added . ");
+                query.endWhere();
+                QStringList orderByBindings;
+                QList<MediaQuery::Order> order;
+                orderByBindings.append("added");
+                order.append(MediaQuery::Descending);
+                query.orderBy(orderByBindings, order);
+
+                Soprano::QueryResultIterator it = query.executeSelect(m_mainModel);
+
+                //Build media list from results
+                while( it.next() ) {
+                    if (m_stop) {
+                        return;
+                    }
+                    MediaItem mediaItem;
+                    QDateTime added = it.binding("added").literal().toDateTime();
+                    Nepomuk::Resource res = Nepomuk::Resource(it.binding(mediaVocabulary.mediaResourceBinding()).uri());
+                    mediaItem = Utilities::mediaItemFromNepomuk(res, m_mediaListProperties.lri);
+                    mediaItem.semanticComment = i18nc("for example, added 3 days ago", "added %1", Utilities::wordsForTimeSince(added));
+                    if (!mediaItem.url.startsWith("nepomuk:/")) {
+                        mediaList.append(mediaItem);
+                    }
+                }
+                m_mediaListProperties.name = i18n("Recently Added");
+                m_mediaListProperties.type = QString("Sources");
+            }
+        }
     }
     
     emit results(m_requestSignature, mediaList, m_mediaListProperties, true, m_subRequestSignature);
@@ -366,3 +434,41 @@ void SemanticsListEngine::run()
     m_requestSignature = QString();
     m_subRequestSignature = QString();
 }
+
+void SemanticsListEngine::addUniqueGenreGroup(QString field, MediaItem mediaItem, QList<MediaItem>* mediaList, int limit)
+{
+    //Resolve and merge raw genres
+    QString resolvedGenre = Utilities::genreFromRawTagGenre(mediaItem.title);
+    bool genreExists = false;
+    for (int i = 0; i < mediaList->count(); i++) {
+        if (Utilities::genreFromRawTagGenre(mediaList->at(i).title) == resolvedGenre) {
+            genreExists = true;
+            mediaItem.title = resolvedGenre;
+            mediaItem.fields["title"] = mediaItem.title;
+            if (field == "rating") {
+                int oldRating = mediaList->at(i).fields["rating"].toInt();
+                int currentRating = mediaItem.fields["rating"].toInt();
+                if (currentRating < oldRating) {
+                    mediaItem.fields["rating"] = oldRating;
+                }
+            } else if (field == "lastPlayed") {
+                QDateTime oldLastPlayed = mediaList->at(i).fields["lastPlayed"].toDateTime();
+                QDateTime currentLastPlayed = mediaItem.fields["lastPlayed"].toDateTime();
+                if (currentLastPlayed < oldLastPlayed) {
+                    mediaItem.fields["lastPlayed"] = oldLastPlayed;
+                    mediaItem.semanticComment = Utilities::wordsForTimeSince(mediaItem.fields["lastPlayed"].toDateTime());
+                }
+            } else if (field == "playCount") {
+                int oldPlayCount = mediaList->at(i).fields["playCount"].toInt();
+                int newPlayCount = oldPlayCount + mediaItem.fields["playCount"].toInt();
+                mediaItem.semanticComment = i18np("played once", "played %1 times", newPlayCount);
+                mediaItem.fields["playCount"] = newPlayCount;
+            }
+            mediaList->replace(i, mediaItem);
+            break;
+        }
+    }
+    if (!genreExists && mediaList->count() < limit) {
+        mediaList->append(mediaItem);
+    }
+}
